help_screen = """usage: kodman [-h] [-v] {run,version} ...

positional arguments:
  {run,version}
    run          Run a command in a new container
    version      Display the kodman version information

options:
  -h, --help     show this help message and exit
  -v, --version  show program's version number and exit

environment variables:
  KODMAN_DEBUG  bool
  KODMAN_SERVICE_ACCOUNT  str"""

hello_world = """Hello from Docker!
This message shows that your installation appears to be working correctly.

To generate this message, Docker took the following steps:
 1. The Docker client contacted the Docker daemon.
 2. The Docker daemon pulled the "hello-world" image from the Docker Hub.
    (amd64)
 3. The Docker daemon created a new container from that image which runs the
    executable that produces the output you are currently reading.
 4. The Docker daemon streamed that output to the Docker client, which sent it
    to your terminal.

To try something more ambitious, you can run an Ubuntu container with:
 $ docker run -it ubuntu bash

Share images, automate workflows, and more with a free Docker ID:
 https://hub.docker.com/

For more examples and ideas, visit:
 https://docs.docker.com/get-started/"""

mount = "test data"

failed_image = "Failed to pull image"

failed_command = 'exec: "bash": executable file not found in $PATH: unknown'
